'use strict';
const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const fs = require('fs-extra');

module.exports = class extends Generator {
  prompting() {
    // Have Yeoman greet the user.
    this.log(
      yosay(`Welcome to the first-class ${chalk.red('generator-test')} generator!`)
    );

    const prompts = [
      {
        type: 'input',
        name: 'projectName',
        message: 'What is your project name?',
        // Defaults to the project's folder name if the input is skipped
        default: this.appname
      },
      {
        type: 'input',
        name: 'databaseName',
        message: 'What is your database name?',
        // Defaults to the project's folder name if the input is skipped
        default: this.appname
      },
      {
        type: 'input',
        name: 'apiVersion',
        message: 'What is your project version?',
        // Defaults to the project's folder name if the input is skipped
        default: '1.0'
      }
    ];

    return this.prompt(prompts).then(props => {
      // To access props later use this.props.someAnswer;
      this.props = props;
    });
  }

  writing() {
    // COPY CONFIGURATION FILES

    // Package.jso file
    this.fs.copyTpl(
      this.templatePath('package.json'),
      this.destinationPath('package.json'),
      {
        projectName: this.props.projectName
      }
    );

    // Environment file
    this.fs.copyTpl(
      this.templatePath('src/config/environments.js'),
      this.destinationPath('src/config/environments.js'),
      {
        projectName: this.props.projectName,
        databaseName: this.props.databaseName
      }
    );

    // Public Routes file
    this.fs.copyTpl(
      this.templatePath('src/config/public_routes.js'),
      this.destinationPath('src/config/public_routes.js'),
      {
        apiVersion: this.props.apiVersion
      }
    );

    // Secrets file
    this.fs.copy(
      this.templatePath('src/config/secrets.js'),
      this.destinationPath('src/config/secrets.js')
    );

    // README file
    this.fs.copy(
      this.templatePath('README.md'),
      this.destinationPath('README.md')
    );

    // COPY APPLICATION FILES

    // Server file
    this.fs.copy(
      this.templatePath('src/server.js'),
      this.destinationPath('src/server.js')
    );

    // Routes file
    this.fs.copy(
      this.templatePath('src/routes.js'),
      this.destinationPath('src/routes.js')
    );

    // Services/Collection file
    this.fs.copy(
      this.templatePath('src/services/collections.js'),
      this.destinationPath('src/services/collections.js')
    );

    // Services/Mailer file
    this.fs.copy(
      this.templatePath('src/services/mailer.js'),
      this.destinationPath('src/services/mailer.js')
    );

    // Middleware auth,local-auth and passport file
    this.fs.copy(
      this.templatePath('src/middlewares/auth.js'),
      this.destinationPath('src/middlewares/auth.js')
    );
    this.fs.copy(
      this.templatePath('src/middlewares/local-auth.js'),
      this.destinationPath('src/middlewares/local-auth.js')
    );
    this.fs.copy(
      this.templatePath('src/middlewares/passport.js'),
      this.destinationPath('src/middlewares/passport.js')
    );

    // Middleware facebook auth file
    this.fs.copy(
      this.templatePath('src/middlewares/facebook.js'),
      this.destinationPath('src/middlewares/facebook.js')
    );

    // Middleware google auth file
    this.fs.copy(
      this.templatePath('src/middlewares/google.js'),
      this.destinationPath('src/middlewares/google.js')
    );

    // Middleware requests file
    this.fs.copy(
      this.templatePath('src/middlewares/requests.js'),
      this.destinationPath('src/middlewares/requests.js')
    );

    // Middleware requests file
    this.fs.copy(
      this.templatePath('src/middlewares/requests.js'),
      this.destinationPath('src/middlewares/requests.js')
    );

    // Mailer Files file
    this.fs.copy(
      this.templatePath('src/mailers'),
      this.destinationPath('src/mailers')
    );

    // Helper Files
    this.fs.copy(
      this.templatePath('src/helpers'),
      this.destinationPath('src/helpers')
    );

    // Api files
    this.fs.copy(
      this.templatePath('src/api'),
      this.destinationPath('src/api')
    );
  }

  install() {
    this.installDependencies();
  }
};
