const router = require('express-promise-router')();
const passport = require('passport');
const passportConf = require('../../middlewares/passport');

const { validateBody, schemas } = require('../../helpers/route-helpers');
const actions = require('./actions');

const passportSignIn = passport.authenticate('local', { session: false });
const passportAuthJWT = passport.authenticate('jwt', { session: false });
const passportGoogle = passport.authenticate('googleToken', { session: false });
const passportFacebook = passport.authenticate('facebookToken', { session: false });

router.route('/signup')
  .post(
    validateBody(schemas.authSchema),
    actions.signUp
  );

router.route('/oauth/google')
  .post(
    passportGoogle,
    actions.googleOAuth
  );

router.route('/oauth/facebook')
  .post(
    passportFacebook,
    actions.facebookOAuth
  );

router.route('/signin')
  .post(
    validateBody(schemas.authSchema),
    passportSignIn,
    actions.signIn
  );

module.exports = router;