const mongoose = require('mongoose');
const bcrypt = require('bcryptjs');
const Schema = mongoose.Schema;

const userSchema = new Schema({
  method: {
    type: String,
    enum: ['local', 'google', 'facebook'],
    required: true
  },
  local: {
    email: {
      type: String,
      lowercase: true
    },
    password: String
  },
  google: {
    id: String,
    email: {
      type: String,
      lowercase: true
    }
  },
  facebook: {
    id: String,
    email: {
      type: String,
      lowercase: true
    }
  },
  active: {
    type: Boolean,
    default: true
  },
  confirmation_link: String,
  profile: {
    avatar: String,
    first_name: String,
    last_name: String,
    country: String,
    birthday: Date,
    phone: String,
    other_phone: String,
    gender: String
  }
});

// Functions called before save
userSchema.pre('save', function(next) {
  if (this.method !== 'local') next();

  let _this = this;
  // Generate a salt
  bcrypt.genSalt(10, function(err, salt) {
    // Generate password hash
    bcrypt.hash(_this.local.password, salt, function(err, hash) {
      //Re-assign hashed version over the normal password
      _this.local.password = hash;
      next();
    });
  })
});

userSchema.methods.isValidPassword = function(newPassword, callback) {
  bcrypt.compare(newPassword, this.local.password, function(err, success) {
    if (err) { throw err; }
    callback(success);
  });
};

userSchema.methods.fullName = function() {
  return `${this.profile.first_name} ${this.profile.last_name}`;
};

const User = mongoose.model('user', userSchema);

module.exports = User;
