const express = require('express');
const api = express();

const logger = require('morgan');
const bodyParser = require('body-parser');
const mongoose = require('mongoose');
const cors = require('cors');

const { requestParser, responseParser, errorHandler } = require('./middlewares/requests');
const { verifyAuth } = require('./middlewares/auth');

const routes = require('./routes');

// Initialize and connect to mongoDB
const environments = require('./config/environments');

if (process.env.NODE_ENV) {
  mongoose.connect(environments[process.env.NODE_ENV].database);
} else {
  mongoose.connect(environments.development.database);
}

// Middlewares
api.use(logger('dev'));
api.use(bodyParser.json());
api.use(bodyParser.urlencoded({ extended: false }));
api.use(cors());

// Routes
api.use(requestParser);
api.use(verifyAuth);
routes.attach(api);
api.use(responseParser);
api.use(errorHandler);

// catch 404 and forward to error handler
api.use(function(req, res, next) {
  let err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// error handler
api.use(function(err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.send(err.message);
});

module.exports = api;